import 'dart:async';

import 'package:circumstellar/circumstellar.dart';

class EntityManager<TObjectId> {
  DiscordClient _client;
  Map<TObjectId, Entity<TObjectId>> _cache = {};

  EntityManager(this._client);

  void updateEntity(
      TObjectId id, Entity<TObjectId> newEntity, String typeName) {
    _cache[id] = newEntity;
  }

  Future<Entity<TObjectId>> getEntity(TObjectId id, String typeName,
      [Map<String, dynamic> newData]) async {
    var cacheResult = _cache[id];
    if (cacheResult != null) {
      return cacheResult;
    }

    if (typeName == 'GuildTextChannel') {
      var gtc = new GuildTextChannel(_client);
      if (newData != null) {
        await gtc.updateModel(newData, _client);
      } else {
        await gtc.updateModel(
            (await _client.rest.executeRoute(
                    ApiRoutes.channels.getChannelWithId(id.toString())))
                .json,
            _client);
      }
      updateEntity(id, (gtc as Entity<TObjectId>), 'GuildTextChannel');
      return _cache[id];
    } else if (typeName == 'Message') {
      return null; // no message caching yet
    } else if (typeName == 'User') {}
    return null;
  }
}
